// Craig S. Kaplan
// Direct manipulation of the control points of a Bezier 
// curve.  Again, this is a lot like DirectManipCircleArray,
// but we don't want to add new points -- we use a fixed 
// array of four control points.

// Radii of all circles.
float rad = 10;

// Define a small Circle class so that we can track information
// about whole circles.
class Circle
{
  float cx;
  float cy;
  color col;
  
  Circle( float cxIn, float cyIn )
  {
    cx = cxIn;
    cy = cyIn;
    
    // Give the circle a random fill colour.
    col = color( random( 255 ), random( 255 ), random( 255 ) );
  }
  
  // Create a method allowing us ask every circle to
  // draw itself.
  void draw()
  {
    fill( col );
    ellipse( cx, cy, 2 * rad, 2 * rad );
  }
  
  // Ask this circle if a given x,y point is inside of it.
  boolean hitTest( float x, float y )
  {
    return dist( cx, cy, x, y ) <= rad;
  }
}

// All the circles in the sketch.
Circle[] circs = {
  new Circle( 100, 250 ),
  new Circle( 200, 250 ),
  new Circle( 300, 250 ),
  new Circle( 400, 250 )
};

// Keep track of an index into the array of the currently
// selected object.  If nothing is selected, use the value
// -1.
int active = -1;

void setup()
{
  size( 500, 500 );
  fill( 200 );
}

void draw()
{
  background( 255 );
  
  noFill();
  stroke( 0 );
  strokeWeight( 15 );
  
  // Draw the Bezier curve
  bezier(
    circs[0].cx, circs[0].cy,
    circs[1].cx, circs[1].cy,
    circs[2].cx, circs[2].cy,
    circs[3].cx, circs[3].cy );
    
  // Then draw all the control points.
  
  strokeWeight( 1 );
  
  // Ask every circle in the array to draw itself.
  for( int idx = 0; idx < circs.length; ++idx ) {
    // Check if we're about to draw the selected circle.
    // If so, highlight its boundary.
    if( active == idx ) {
      stroke( 255, 0, 0 );
    } else {
      stroke( 0 );
    }
    
    circs[idx].draw();
  }
}

void mousePressed()
{
  // Hit test every circle in the array in reverse order.
  // If you find one that passes, make it the active circle.
  for( int idx = circs.length - 1; idx >= 0; --idx ) {
    if( circs[idx].hitTest( mouseX, mouseY ) ) {
      active = idx;
      // Use "return" to break out of the function, because
      // we found the answer and there's nothing left to do.
      return;
    }
  }
  
  // Don't add new circles here.  We're done.
}

void mouseDragged()
{
  if( active >= 0 ) {
    Circle circ = circs[active];
    circ.cx += mouseX - pmouseX;
    circ.cy += mouseY - pmouseY;
  }
}

void mouseReleased()
{
  active = -1;
}