// Craig S. Kaplan
// Direct manipulation of the vertices of a polygon.  This
// is basically the same as direct manipulation of an array 
// of circles, but we draw a polygon connecting the circles.

// Radii of all circles.
float rad = 10;

// Define a small Circle class so that we can track information
// about whole circles.
class Circle
{
  float cx;
  float cy;
  color col;
  
  Circle( float cxIn, float cyIn )
  {
    cx = cxIn;
    cy = cyIn;
    
    // Give the circle a random fill colour.
    col = color( random( 255 ), random( 255 ), random( 255 ) );
  }
  
  // Create a method allowing us ask every circle to
  // draw itself.
  void draw()
  {
    fill( col );
    ellipse( cx, cy, 2 * rad, 2 * rad );
  }
  
  // Ask this circle if a given x,y point is inside of it.
  boolean hitTest( float x, float y )
  {
    return dist( cx, cy, x, y ) <= rad;
  }
}

// All the circles in the sketch.
Circle[] circs;

// Keep track of an index into the array of the currently
// selected object.  If nothing is selected, use the value
// -1.
int active = -1;

void setup()
{
  size( 500, 500 );
  fill( 200 );
  
  circs = new Circle[0];
}

void draw()
{
  background( 255 );
  
  fill( 200 );
  stroke( 0 );
  strokeWeight( 2 );
  
  beginShape();
  for( int idx = 0; idx < circs.length; ++idx ) {
    Circle circ = circs[idx];
    vertex( circ.cx, circ.cy );
  }
  endShape( CLOSE );
  
  strokeWeight( 1 );
  
  // Ask every circle in the array to draw itself.
  for( int idx = 0; idx < circs.length; ++idx ) {
    // Check if we're about to draw the selected circle.
    // If so, highlight its boundary.
    if( active == idx ) {
      stroke( 255, 0, 0 );
    } else {
      stroke( 0 );
    }
    
    circs[idx].draw();
  }
}

void mousePressed()
{
  // Hit test every circle in the array in reverse order.
  // If you find one that passes, make it the active circle.
  for( int idx = circs.length - 1; idx >= 0; --idx ) {
    if( circs[idx].hitTest( mouseX, mouseY ) ) {
      active = idx;
      // Use "return" to break out of the function, because
      // we found the answer and there's nothing left to do.
      return;
    }
  }
  
  // If we got this far, it's because all the hit test failed.
  // Let's use that as an opportunity to add a new circle.
  Circle ncirc = new Circle( mouseX, mouseY );
  circs = (Circle[])append( circs, ncirc );
  active = circs.length - 1;
}

void mouseDragged()
{
  if( active >= 0 ) {
    Circle circ = circs[active];
    circ.cx += mouseX - pmouseX;
    circ.cy += mouseY - pmouseY;
  }
}

void mouseReleased()
{
  active = -1;
}