// Craig S. Kaplan
// A demonstration of the use of ControlP5.  Draw pretty
// mathematical diagrams based in a simulation of a harmonograph:
// http://www.anitachowdry.com/iron-genie/4581902678
// It's sort of like a spirograph on steroids.  Create sliders
// for the 16 numbers that control the harmonograph shape.

// You don't need to understand all the details of how the
// shape is drawn.  It's really just a demonstration of how
// to set up all the sliders.

import controlP5.*;

ControlP5 ui;

float[] A = { 0, 0, 0, 0 };
float[] f = { 0, 0, 0, 0 };
float[] d = { 0, 0, 0, 0 };
float[] p = { 0, 0, 0, 0 };

float eval( float t, float A, float f, float p, float d )
{
  return A * sin( t * f + p ) * exp( -d * t );
}

float evalX( float t )
{
  return eval( t, A[0], f[0], p[0], d[0] ) +
    eval( t, A[1], f[1], p[1], d[1] );
}

float evalY( float t )
{
  return eval( t, A[2], f[2], p[2], d[2] ) +
    eval( t, A[3], f[3], p[3], d[3] );
}

void setup()
{
  size( 800, 600 );
  ui = new ControlP5( this );

  for ( int idx = 0; idx < 4; ++idx ) {
    ui.addSlider( "A" + (idx+1) )
      .setPosition( 10 + 175 * idx, 5 )
      .setRange( 0, 300 )
      .setSize( 150, 20 );
    ui.addSlider( "f" + (idx+1) )
      .setPosition( 10 + 175 * idx, 30 )
      .setRange( 0, 20 )
      .setSize( 150, 20 );
    ui.addSlider( "p" + (idx+1) )
      .setPosition( 10 + 175 * idx, 55 )
      .setRange( 0, TWO_PI )
      .setSize( 150, 20 );
    ui.addSlider( "d" + (idx+1) )
      .setPosition( 10 + 175 * idx, 80 )
      .setRange( 0, 0.1 )
      .setSize( 150, 20 );
  }

  drawDiagram();
}

void drawDiagram()
{
  background( 50 );
  noFill();
  stroke( 255, 255, 255, 80 );
  strokeWeight( 0.5 );

  float lastx = 0.0;
  float lasty = 0.0;
  for ( int idx = 0; idx < 10000; ++idx ) {
    float t = idx / 100.0;
    float curx = width/2 + evalX( t );
    float cury = height/2 + evalY( t );
    if ( idx > 0 ) {
      line( lastx, lasty, curx, cury );
    }

    lastx = curx;
    lasty = cury;
  }
}

void draw()
{
}

void controlEvent( ControlEvent ce )
{
  // Do a bit complicated reverse engineering on the
  // name of the slider to figure out what we're
  // editing.
  
  String name = ce.getName();
  float[] arr = A;
  if ( name.charAt( 0 ) == 'A' ) { 
    arr = A;
  } else if ( name.charAt( 0 ) == 'f' ) { 
    arr = f;
  } else if ( name.charAt( 0 ) == 'p' ) { 
    arr = p;
  } else if ( name.charAt( 0 ) == 'd' ) { 
    arr = d;
  }

  int idx = int( name.substring( 1 ) ) - 1;
  arr[idx] = ce.getValue();
  drawDiagram();
}