// Craig S. Kaplan
// A second demonstration of direct manipulation: drag a 
// circle and a square separately.  This gives us more stuff
// to keep track of.

// The coordinates of the centre of the circle.
float ex = 150;
float ey = 150;

// The radius of the circle.
float rad = 100;

// The top-left corner of the square.
float ax = 350;
float ay = 350;

// The edge length of the square.
float w = 100;

// Because we have two objects, we need to keep track
// of what's selected.  It could be nothing, or one of
// these two objects.
boolean ellipse_active = false;
boolean square_active = false;

void setup()
{
  size( 500, 500 );
  fill( 200 );
}

void draw()
{
  background( 255 );
  
  // Draw the square, then draw the circle on top of it.
  rect( ax, ay, w, w );
  ellipse( ex, ey, 2*rad, 2*rad );
}

void mousePressed()
{
  // Because we drew the circle on top the square, we now
  // hit test in reverse order, corresponding to our intuition
  // for the circle lying on top of the square.
  if ( dist( mouseX, mouseY, ex, ey ) <= rad ) {
    ellipse_active = true;
  } else if ( (mouseX >= ax) && (mouseX <= (ax+w))
    && (mouseY >= ay) && (mouseY <= (ay+w)) ) {
    square_active = true;
  }
}

void mouseDragged()
{
  // Move either the circle or the square, if 
  // needed.
  
  if( ellipse_active ) {
    ex += (mouseX-pmouseX);
    ey += (mouseY-pmouseY);
  } else if( square_active ) {
    ax += (mouseX-pmouseX);
    ay += (mouseY-pmouseY);
  }
}

void mouseReleased()
{
  ellipse_active = false;
  square_active = false;
}