// Craig S. Kaplan
// A variation of the SquareEllipse sketch that handles
// tracking the active object slightly differently, just
// to show some options.

// The coordinates of the centre of the circle.
float ex = 150;
float ey = 150;

// The radius of the circle.
float rad = 100;

// The top-left corner of the square.
float ax = 350;
float ay = 350;

// The edge length of the square.
float w = 100;

// Define some magic constants to remember what's selected.
int SEL_NOTHING = 0;
int SEL_CIRCLE = 1;
int SEL_SQUARE = 2;

// Now have a variable that will always be equal to one of
// the above options.
int active = SEL_NOTHING;

void setup()
{
  size( 500, 500 );
  fill( 200 );
}

void draw()
{
  background( 255 );
  
  // Draw the square, then draw the circle on top of it.
  rect( ax, ay, w, w );
  ellipse( ex, ey, 2*rad, 2*rad );
}

void mousePressed()
{
  // Because we drew the circle on top the square, we now
  // hit test in reverse order, corresponding to our intuition
  // for the circle lying on top of the square.
  if ( dist( mouseX, mouseY, ex, ey ) <= rad ) {
    active = SEL_CIRCLE;
  } else if ( (mouseX >= ax) && (mouseX <= (ax+w))
    && (mouseY >= ay) && (mouseY <= (ay+w)) ) {
    active = SEL_SQUARE;
  }
}

void mouseDragged()
{
  // Move either the circle or the square, if 
  // needed.
  
  if( active == SEL_CIRCLE ) {
    ex += (mouseX-pmouseX);
    ey += (mouseY-pmouseY);
  } else if( active == SEL_SQUARE ) {
    ax += (mouseX-pmouseX);
    ay += (mouseY-pmouseY);
  }
}

void mouseReleased()
{
  active = SEL_NOTHING;
}